﻿using System;

using Prism;
using Prism.Commands;
using Prism.Mvvm;

using TestLibrary;

namespace TestModule.ViewModels
{
    /// <summary>
    /// 탭 뷰 모델
    /// </summary>
    public class TabViewModel : BindableBase, IActiveAware
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 활성화 여부 변경시 이벤트 - IsActiveChanged

        /// <summary>
        /// 활성화 여부 변경시 이벤트
        /// </summary>
        public event EventHandler IsActiveChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 어플리케이션 명령
        /// </summary>
        private IApplicationCommand applicationCommand;

        /// <summary>
        /// 제목
        /// </summary>
        private string title;

        /// <summary>
        /// 업데이트 가능 여부
        /// </summary>
        private bool canUpdate = true;

        /// <summary>
        /// 업데이트 텍스트
        /// </summary>
        private string updatedText;

        /// <summary>
        /// 활성화 여부
        /// </summary>
        private bool isActive;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 제목 - Title

        /// <summary>
        /// 제목
        /// </summary>
        public string Title
        {
            get
            {
                return this.title;
            }
            set
            {
                SetProperty(ref this.title, value);
            }
        }

        #endregion
        #region 업데이트 가능 여부 - CanUpdate

        /// <summary>
        /// 업데이트 가능 여부
        /// </summary>
        public bool CanUpdate
        {
            get
            {
                return this.canUpdate;
            }
            set
            {
                SetProperty(ref this.canUpdate, value);
            }
        }

        #endregion
        #region 업데이트 텍스트 - UpdatedText

        /// <summary>
        /// 업데이트 텍스트
        /// </summary>
        public string UpdatedText
        {
            get
            {
                return this.updatedText;
            }
            set
            {
                SetProperty(ref this.updatedText, value);
            }
        }

        #endregion
        #region 업데이트 명령 - UpdateCommand

        /// <summary>
        /// 업데이트 명령
        /// </summary>
        public DelegateCommand UpdateCommand { get; private set; }

        #endregion
        #region 활성화 여부 - IsActive

        /// <summary>
        /// 활성화 여부
        /// </summary>
        public bool IsActive
        {
            get
            {
                return this.isActive;
            }
            set
            {
                this.isActive = value;

                FireIsActiveChangedEvent();
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - TabViewModel(applicationCommand)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="applicationCommand">어플리케이션 명령</param>
        public TabViewModel(IApplicationCommand applicationCommand)
        {
            this.applicationCommand = applicationCommand;

            UpdateCommand = new DelegateCommand(Update).ObservesCanExecute(() => CanUpdate);

            this.applicationCommand.SaveCommand.RegisterCommand(UpdateCommand);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 업데이트하기 - Update()

        /// <summary>
        /// 업데이트하기
        /// </summary>
        private void Update()
        {
            UpdatedText = $"Updated : {DateTime.Now}";
        }

        #endregion
        #region 활성화 여부 변경시 이벤트 발생시키기 - FireIsActiveChangedEvent()

        /// <summary>
        /// 활성화 여부 변경시 이벤트 발생시키기
        /// </summary>
        private void FireIsActiveChangedEvent()
        {
            UpdateCommand.IsActive = IsActive;

            IsActiveChanged?.Invoke(this, new EventArgs());
        }

        #endregion
    }
}